@@ -1,7 +1,11 @@
+# coding=utf-8
+
+from __future__ import division
+import time
 import struct
 
 
-class Command(object):
+class ProtocolCode(object):
     # BASIC
     HEADER = 0xFE
     FOOTER = 0xFA
@@ -83,7 +87,10 @@ class DataProcessor(object):
         return struct.pack("b", data)
 
     def _encode_int16(self, data):
-        return list(struct.pack(">h", data))
+        if isinstance(data, int):
+            return list(struct.pack(">h", data))
+        else:
+            return sum([list(struct.pack('>h', elem)) for elem in data], [])
 
     def _decode_int8(self, data):
         return struct.unpack("b", data)[0]
@@ -91,21 +98,22 @@ class DataProcessor(object):
     def _decode_int16(self, data):
         return struct.unpack(">h", data)[0]
 
-    def _angle_to_int(self, angle):
+    def _angle2int(self, angle):
         return int(angle * 100)
 
-    def _coord_to_int(self, coord):
+    def _coord2int(self, coord):
         return int(coord * 10)
 
-    def _int_to_angle(self, _int):
+    def _int2angle(self, _int):
         return round(_int / 100.0, 3)
 
-    def _int_to_coord(self, _int):
+    def _int2coord(self, _int):
         return round(_int / 10.0, 2)
 
     def _flatten(self, _list):
         return sum(
-            ([x] if not isinstance(x, list) else self._flatten(x) for x in _list), []
+            ([x] if not isinstance(x, list) else self._flatten(x)
+             for x in _list), []
         )
 
     def _process_data_command(self, args):
@@ -114,13 +122,14 @@ class DataProcessor(object):
 
         return self._flatten(
             [
-                [self._encode_int16(int(i)) for i in x] if isinstance(x, list) else x
+                [self._encode_int16(int(i))
+                 for i in x] if isinstance(x, list) else x
                 for x in args
             ]
         )
 
     def _is_frame_header(self, data, pos):
-        return data[pos] == Command.HEADER and data[pos + 1] == Command.HEADER
+        return data[pos] == ProtocolCode.HEADER and data[pos + 1] == ProtocolCode.HEADER
 
     def _process_received(self, data, genre):
         if not data:
@@ -131,27 +140,32 @@ class DataProcessor(object):
         # Get valid header: 0xfe0xfe
         for idx in range(data_len - 1):
             if self._is_frame_header(data, idx):
-                break
+                data_len = data[idx + 2] - 2
+                if data_len > 0:
+                    break
         else:
             return []
-
-        data_len = data[idx + 2] - 2
-
+        unique_data = [ProtocolCode.GET_BASIC_INPUT,
+                       ProtocolCode.GET_DIGITAL_INPUT]
         # compare send header and received header
         cmd_id = data[idx + 3]
         if cmd_id != genre:
             return []
-        data_pos = idx + 4
-        valid_data = data[data_pos : data_pos + data_len]
+        if cmd_id in unique_data:
+            data_pos = idx + 5
+            data_len -= 1
+        else:
+            data_pos = idx + 4
+        valid_data = data[data_pos: data_pos + data_len]
 
         # process valid data
         res = []
-        if data_len == 12:
+        if data_len == 12 or data_len == 8:
             for idx in range(0, len(valid_data), 2):
-                one = valid_data[idx : idx + 2]
+                one = valid_data[idx: idx + 2]
                 res.append(self._decode_int16(one))
         elif data_len == 2:
-            if genre in [Command.IS_SERVO_ENABLE]:
+            if genre in [ProtocolCode.IS_SERVO_ENABLE]:
                 return [self._decode_int8(valid_data[1:2])]
             res.append(self._decode_int16(valid_data))
         else:
@@ -160,3 +174,21 @@ class DataProcessor(object):
 
     def _process_single(self, data):
         return data[0] if data else -1
+
+
+def write(self, command):
+    self.log.debug("_write: {}".format(command))
+
+    self._serial_port.write(command)
+    self._serial_port.flush()
+    time.sleep(0.05)
+
+
+def read(self):
+    if self._serial_port.inWaiting() > 0:
+        data = self._serial_port.read(self._serial_port.inWaiting())
+        self.log.debug("_read: {}".format(data))
+    else:
+        self.log.debug("_read: no data can be read")
+        data = None
+    return data
